@@ -1,3 +1,4 @@
+#!/bin/sh
 apt-get update && apt-get install -y --no-install-recommends \
   automake \
   bison \
@@ -17,9 +18,10 @@ apt-get update && apt-get install -y --no-install-recommends \
   libssl-dev \
   libtool-bin \
   make \
+  ninja-build \
   patch \
   pkg-config \
-  python2.7 \
+  python3 \
   sudo \
   texinfo \
   unzip \
