@@ -1,4 +1,4 @@
-use std::collections::BTreeMap;
+use std::collections::{BTreeMap, HashMap};
 use std::convert::TryFrom;
 use std::ffi::OsStr;
 use std::fmt;
@@ -30,13 +30,13 @@ use crate::passes::{self, Condition, DefaultPassOption};
 use crate::theme;
 
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
-pub enum OutputFormat {
+crate enum OutputFormat {
     Json,
     Html,
 }
 
 impl OutputFormat {
-    pub fn is_json(&self) -> bool {
+    crate fn is_json(&self) -> bool {
         match self {
             OutputFormat::Json => true,
             _ => false,
@@ -58,93 +58,96 @@ impl TryFrom<&str> for OutputFormat {
 
 /// Configuration options for rustdoc.
 #[derive(Clone)]
-pub struct Options {
+crate struct Options {
     // Basic options / Options passed directly to rustc
     /// The crate root or Markdown file to load.
-    pub input: PathBuf,
+    crate input: PathBuf,
     /// The name of the crate being documented.
-    pub crate_name: Option<String>,
+    crate crate_name: Option<String>,
     /// Whether or not this is a proc-macro crate
-    pub proc_macro_crate: bool,
+    crate proc_macro_crate: bool,
     /// How to format errors and warnings.
-    pub error_format: ErrorOutputType,
+    crate error_format: ErrorOutputType,
     /// Library search paths to hand to the compiler.
-    pub libs: Vec<SearchPath>,
+    crate libs: Vec<SearchPath>,
     /// Library search paths strings to hand to the compiler.
-    pub lib_strs: Vec<String>,
+    crate lib_strs: Vec<String>,
     /// The list of external crates to link against.
-    pub externs: Externs,
+    crate externs: Externs,
     /// The list of external crates strings to link against.
-    pub extern_strs: Vec<String>,
+    crate extern_strs: Vec<String>,
     /// List of `cfg` flags to hand to the compiler. Always includes `rustdoc`.
-    pub cfgs: Vec<String>,
+    crate cfgs: Vec<String>,
     /// Codegen options to hand to the compiler.
-    pub codegen_options: CodegenOptions,
+    crate codegen_options: CodegenOptions,
     /// Codegen options strings to hand to the compiler.
-    pub codegen_options_strs: Vec<String>,
+    crate codegen_options_strs: Vec<String>,
     /// Debugging (`-Z`) options to pass to the compiler.
-    pub debugging_options: DebuggingOptions,
+    crate debugging_opts: DebuggingOptions,
     /// Debugging (`-Z`) options strings to pass to the compiler.
-    pub debugging_options_strs: Vec<String>,
+    crate debugging_opts_strs: Vec<String>,
     /// The target used to compile the crate against.
-    pub target: TargetTriple,
+    crate target: TargetTriple,
     /// Edition used when reading the crate. Defaults to "2015". Also used by default when
     /// compiling doctests from the crate.
-    pub edition: Edition,
+    crate edition: Edition,
     /// The path to the sysroot. Used during the compilation process.
-    pub maybe_sysroot: Option<PathBuf>,
+    crate maybe_sysroot: Option<PathBuf>,
     /// Lint information passed over the command-line.
-    pub lint_opts: Vec<(String, Level)>,
+    crate lint_opts: Vec<(String, Level)>,
     /// Whether to ask rustc to describe the lints it knows. Practically speaking, this will not be
     /// used, since we abort if we have no input file, but it's included for completeness.
-    pub describe_lints: bool,
+    crate describe_lints: bool,
     /// What level to cap lints at.
-    pub lint_cap: Option<Level>,
+    crate lint_cap: Option<Level>,
 
     // Options specific to running doctests
     /// Whether we should run doctests instead of generating docs.
-    pub should_test: bool,
+    crate should_test: bool,
     /// List of arguments to pass to the test harness, if running tests.
-    pub test_args: Vec<String>,
+    crate test_args: Vec<String>,
     /// Optional path to persist the doctest executables to, defaults to a
     /// temporary directory if not set.
-    pub persist_doctests: Option<PathBuf>,
+    crate persist_doctests: Option<PathBuf>,
     /// Runtool to run doctests with
-    pub runtool: Option<String>,
+    crate runtool: Option<String>,
     /// Arguments to pass to the runtool
-    pub runtool_args: Vec<String>,
+    crate runtool_args: Vec<String>,
     /// Whether to allow ignoring doctests on a per-target basis
     /// For example, using ignore-foo to ignore running the doctest on any target that
     /// contains "foo" as a substring
-    pub enable_per_target_ignores: bool,
+    crate enable_per_target_ignores: bool,
 
     /// The path to a rustc-like binary to build tests with. If not set, we
     /// default to loading from $sysroot/bin/rustc.
-    pub test_builder: Option<PathBuf>,
+    crate test_builder: Option<PathBuf>,
 
     // Options that affect the documentation process
     /// The selected default set of passes to use.
     ///
     /// Be aware: This option can come both from the CLI and from crate attributes!
-    pub default_passes: DefaultPassOption,
+    crate default_passes: DefaultPassOption,
     /// Any passes manually selected by the user.
     ///
     /// Be aware: This option can come both from the CLI and from crate attributes!
-    pub manual_passes: Vec<String>,
+    crate manual_passes: Vec<String>,
     /// Whether to display warnings during doc generation or while gathering doctests. By default,
     /// all non-rustdoc-specific lints are allowed when generating docs.
-    pub display_warnings: bool,
+    crate display_warnings: bool,
     /// Whether to run the `calculate-doc-coverage` pass, which counts the number of public items
     /// with and without documentation.
-    pub show_coverage: bool,
+    crate show_coverage: bool,
 
     // Options that alter generated documentation pages
     /// Crate version to note on the sidebar of generated docs.
-    pub crate_version: Option<String>,
+    crate crate_version: Option<String>,
     /// Collected options specific to outputting final pages.
-    pub render_options: RenderOptions,
+    crate render_options: RenderOptions,
     /// Output format rendering (used only for "show-coverage" option for the moment)
-    pub output_format: Option<OutputFormat>,
+    crate output_format: Option<OutputFormat>,
+    /// If this option is set to `true`, rustdoc will only run checks and not generate
+    /// documentation.
+    crate run_check: bool,
 }
 
 impl fmt::Debug for Options {
@@ -185,91 +188,96 @@ impl fmt::Debug for Options {
             .field("runtool", &self.runtool)
             .field("runtool_args", &self.runtool_args)
             .field("enable-per-target-ignores", &self.enable_per_target_ignores)
+            .field("run_check", &self.run_check)
             .finish()
     }
 }
 
 /// Configuration options for the HTML page-creation process.
 #[derive(Clone, Debug)]
-pub struct RenderOptions {
+crate struct RenderOptions {
     /// Output directory to generate docs into. Defaults to `doc`.
-    pub output: PathBuf,
+    crate output: PathBuf,
     /// External files to insert into generated pages.
-    pub external_html: ExternalHtml,
+    crate external_html: ExternalHtml,
     /// A pre-populated `IdMap` with the default headings and any headings added by Markdown files
     /// processed by `external_html`.
-    pub id_map: IdMap,
+    crate id_map: IdMap,
     /// If present, playground URL to use in the "Run" button added to code samples.
     ///
     /// Be aware: This option can come both from the CLI and from crate attributes!
-    pub playground_url: Option<String>,
+    crate playground_url: Option<String>,
     /// Whether to sort modules alphabetically on a module page instead of using declaration order.
     /// `true` by default.
     //
     // FIXME(misdreavus): the flag name is `--sort-modules-by-appearance` but the meaning is
     // inverted once read.
-    pub sort_modules_alphabetically: bool,
+    crate sort_modules_alphabetically: bool,
     /// List of themes to extend the docs with. Original argument name is included to assist in
     /// displaying errors if it fails a theme check.
-    pub themes: Vec<StylePath>,
+    crate themes: Vec<StylePath>,
     /// If present, CSS file that contains rules to add to the default CSS.
-    pub extension_css: Option<PathBuf>,
+    crate extension_css: Option<PathBuf>,
     /// A map of crate names to the URL to use instead of querying the crate's `html_root_url`.
-    pub extern_html_root_urls: BTreeMap<String, String>,
+    crate extern_html_root_urls: BTreeMap<String, String>,
+    /// A map of the default settings (values are as for DOM storage API). Keys should lack the
+    /// `rustdoc-` prefix.
+    crate default_settings: HashMap<String, String>,
     /// If present, suffix added to CSS/JavaScript files when referencing them in generated pages.
-    pub resource_suffix: String,
+    crate resource_suffix: String,
     /// Whether to run the static CSS/JavaScript through a minifier when outputting them. `true` by
     /// default.
     //
     // FIXME(misdreavus): the flag name is `--disable-minification` but the meaning is inverted
     // once read.
-    pub enable_minification: bool,
+    crate enable_minification: bool,
     /// Whether to create an index page in the root of the output directory. If this is true but
     /// `enable_index_page` is None, generate a static listing of crates instead.
-    pub enable_index_page: bool,
+    crate enable_index_page: bool,
     /// A file to use as the index page at the root of the output directory. Overrides
     /// `enable_index_page` to be true if set.
-    pub index_page: Option<PathBuf>,
+    crate index_page: Option<PathBuf>,
     /// An optional path to use as the location of static files. If not set, uses combinations of
     /// `../` to reach the documentation root.
-    pub static_root_path: Option<String>,
+    crate static_root_path: Option<String>,
 
     // Options specific to reading standalone Markdown files
     /// Whether to generate a table of contents on the output file when reading a standalone
     /// Markdown file.
-    pub markdown_no_toc: bool,
+    crate markdown_no_toc: bool,
     /// Additional CSS files to link in pages generated from standalone Markdown files.
-    pub markdown_css: Vec<String>,
+    crate markdown_css: Vec<String>,
     /// If present, playground URL to use in the "Run" button added to code samples generated from
     /// standalone Markdown files. If not present, `playground_url` is used.
-    pub markdown_playground_url: Option<String>,
+    crate markdown_playground_url: Option<String>,
     /// If false, the `select` element to have search filtering by crates on rendered docs
     /// won't be generated.
-    pub generate_search_filter: bool,
+    crate generate_search_filter: bool,
     /// Document items that have lower than `pub` visibility.
-    pub document_private: bool,
+    crate document_private: bool,
     /// Document items that have `doc(hidden)`.
-    pub document_hidden: bool,
+    crate document_hidden: bool,
+    crate unstable_features: rustc_feature::UnstableFeatures,
 }
 
 /// Temporary storage for data obtained during `RustdocVisitor::clean()`.
 /// Later on moved into `CACHE_KEY`.
 #[derive(Default, Clone)]
-pub struct RenderInfo {
-    pub inlined: FxHashSet<DefId>,
-    pub external_paths: crate::core::ExternalPaths,
-    pub exact_paths: FxHashMap<DefId, Vec<String>>,
-    pub access_levels: AccessLevels<DefId>,
-    pub deref_trait_did: Option<DefId>,
-    pub deref_mut_trait_did: Option<DefId>,
-    pub owned_box_did: Option<DefId>,
-    pub output_format: Option<OutputFormat>,
+crate struct RenderInfo {
+    crate inlined: FxHashSet<DefId>,
+    crate external_paths: crate::core::ExternalPaths,
+    crate exact_paths: FxHashMap<DefId, Vec<String>>,
+    crate access_levels: AccessLevels<DefId>,
+    crate deref_trait_did: Option<DefId>,
+    crate deref_mut_trait_did: Option<DefId>,
+    crate owned_box_did: Option<DefId>,
+    crate output_format: Option<OutputFormat>,
 }
 
 impl Options {
     /// Parses the given command-line for options. If an error message or other early-return has
     /// been printed, returns `Err` with the exit code.
-    pub fn from_matches(matches: &getopts::Matches) -> Result<Options, i32> {
+    crate fn from_matches(matches: &getopts::Matches) -> Result<Options, i32> {
         // Check for unstable options.
         nightly_options::check_nightly_options(&matches, &opts());
 
@@ -292,7 +300,7 @@ impl Options {
                 println_condition(p.condition);
             }
 
-            if nightly_options::is_nightly_build() {
+            if nightly_options::match_is_nightly_build(matches) {
                 println!("\nPasses run with `--show-coverage`:");
                 for p in passes::COVERAGE_PASSES {
                     print!("{:>20}", p.pass.name);
@@ -318,9 +326,9 @@ impl Options {
         let error_format = config::parse_error_format(&matches, color, json_rendered);
 
         let codegen_options = build_codegen_options(matches, error_format);
-        let debugging_options = build_debugging_options(matches, error_format);
+        let debugging_opts = build_debugging_options(matches, error_format);
 
-        let diag = new_handler(error_format, None, &debugging_options);
+        let diag = new_handler(error_format, None, &debugging_opts);
 
         // check for deprecated options
         check_deprecated_options(&matches, &diag);
@@ -365,7 +373,7 @@ impl Options {
             .iter()
             .map(|s| SearchPath::from_cli_opt(s, error_format))
             .collect();
-        let externs = parse_externs(&matches, &debugging_options, error_format);
+        let externs = parse_externs(&matches, &debugging_opts, error_format);
         let extern_html_root_urls = match parse_extern_html_roots(&matches) {
             Ok(ex) => ex,
             Err(err) => {
@@ -374,6 +382,29 @@ impl Options {
             }
         };
 
+        let default_settings: Vec<Vec<(String, String)>> = vec![
+            matches
+                .opt_str("default-theme")
+                .iter()
+                .map(|theme| {
+                    vec![
+                        ("use-system-theme".to_string(), "false".to_string()),
+                        ("theme".to_string(), theme.to_string()),
+                    ]
+                })
+                .flatten()
+                .collect(),
+            matches
+                .opt_strs("default-setting")
+                .iter()
+                .map(|s| match s.split_once('=') {
+                    None => (s.clone(), "true".to_string()),
+                    Some((k, v)) => (k.to_string(), v.to_string()),
+                })
+                .collect(),
+        ];
+        let default_settings = default_settings.into_iter().flatten().collect();
+
         let test_args = matches.opt_strs("test-args");
         let test_args: Vec<String> =
             test_args.iter().flat_map(|s| s.split_whitespace()).map(|s| s.to_string()).collect();
@@ -416,14 +447,12 @@ impl Options {
                     return Err(1);
                 } else if !ret.is_empty() {
                     diag.struct_warn(&format!(
-                        "theme file \"{}\" is missing CSS rules from the \
-                                               default theme",
+                        "theme file \"{}\" is missing CSS rules from the default theme",
                         theme_s
                     ))
                     .warn("the theme may appear incorrect when loaded")
                     .help(&format!(
-                        "to see what rules are missing, call `rustdoc \
-                                        --check-theme \"{}\"`",
+                        "to see what rules are missing, call `rustdoc  --check-theme \"{}\"`",
                         theme_s
                     ))
                     .emit();
@@ -452,6 +481,7 @@ impl Options {
             &matches.opt_strs("html-after-content"),
             &matches.opt_strs("markdown-before-content"),
             &matches.opt_strs("markdown-after-content"),
+            nightly_options::match_is_nightly_build(&matches),
             &diag,
             &mut id_map,
             edition,
@@ -508,7 +538,9 @@ impl Options {
         let output_format = match matches.opt_str("output-format") {
             Some(s) => match OutputFormat::try_from(s.as_str()) {
                 Ok(o) => {
-                    if o.is_json() && !(show_coverage || nightly_options::is_nightly_build()) {
+                    if o.is_json()
+                        && !(show_coverage || nightly_options::match_is_nightly_build(matches))
+                    {
                         diag.struct_err("json output format isn't supported for doc generation")
                             .emit();
                         return Err(1);
@@ -546,7 +578,7 @@ impl Options {
         let persist_doctests = matches.opt_str("persist-doctests").map(PathBuf::from);
         let test_builder = matches.opt_str("test-builder").map(PathBuf::from);
         let codegen_options_strs = matches.opt_strs("C");
-        let debugging_options_strs = matches.opt_strs("Z");
+        let debugging_opts_strs = matches.opt_strs("Z");
         let lib_strs = matches.opt_strs("L");
         let extern_strs = matches.opt_strs("extern");
         let runtool = matches.opt_str("runtool");
@@ -554,12 +586,12 @@ impl Options {
         let enable_per_target_ignores = matches.opt_present("enable-per-target-ignores");
         let document_private = matches.opt_present("document-private-items");
         let document_hidden = matches.opt_present("document-hidden-items");
+        let run_check = matches.opt_present("check");
 
         let (lint_opts, describe_lints, lint_cap) = get_cmd_lint_options(matches, error_format);
 
         Ok(Options {
             input,
-            crate_name,
             proc_macro_crate,
             error_format,
             libs,
@@ -569,8 +601,8 @@ impl Options {
             cfgs,
             codegen_options,
             codegen_options_strs,
-            debugging_options,
-            debugging_options_strs,
+            debugging_opts,
+            debugging_opts_strs,
             target,
             edition,
             maybe_sysroot,
@@ -589,6 +621,7 @@ impl Options {
             runtool_args,
             enable_per_target_ignores,
             test_builder,
+            run_check,
             render_options: RenderOptions {
                 output,
                 external_html,
@@ -598,6 +631,7 @@ impl Options {
                 themes,
                 extension_css,
                 extern_html_root_urls,
+                default_settings,
                 resource_suffix,
                 enable_minification,
                 enable_index_page,
@@ -609,13 +643,17 @@ impl Options {
                 generate_search_filter,
                 document_private,
                 document_hidden,
+                unstable_features: rustc_feature::UnstableFeatures::from_environment(
+                    crate_name.as_deref(),
+                ),
             },
+            crate_name,
             output_format,
         })
     }
 
     /// Returns `true` if the file given as `self.input` is a Markdown file.
-    pub fn markdown_input(&self) -> bool {
+    crate fn markdown_input(&self) -> bool {
         self.input.extension().map_or(false, |e| e == "md" || e == "markdown")
     }
 }
@@ -627,7 +665,8 @@ fn check_deprecated_options(matches: &getopts::Matches, diag: &rustc_errors::Han
     for flag in deprecated_flags.iter() {
         if matches.opt_present(flag) {
             if *flag == "output-format"
-                && (matches.opt_present("show-coverage") || nightly_options::is_nightly_build())
+                && (matches.opt_present("show-coverage")
+                    || nightly_options::match_is_nightly_build(matches))
             {
                 continue;
             }
@@ -665,11 +704,9 @@ fn parse_extern_html_roots(
 ) -> Result<BTreeMap<String, String>, &'static str> {
     let mut externs = BTreeMap::new();
     for arg in &matches.opt_strs("extern-html-root-url") {
-        let mut parts = arg.splitn(2, '=');
-        let name = parts.next().ok_or("--extern-html-root-url must not be empty")?;
-        let url = parts.next().ok_or("--extern-html-root-url must be of the form name=url")?;
+        let (name, url) =
+            arg.split_once('=').ok_or("--extern-html-root-url must be of the form name=url")?;
         externs.insert(name.to_string(), url.to_string());
     }
-
     Ok(externs)
 }
