@@ -8,46 +8,52 @@
 #![deny(const_err)]
 
 use std::fmt;
-use std::{i8, i16, i32, i64, isize};
-use std::{u8, u16, u32, u64, usize};
 
-const VALS_I8: (i8,) = //~ ERROR any use of this value will cause an error
+const VALS_I8: (i8,) =
     (
      i8::MAX + 1,
      );
+ //~^^ ERROR any use of this value will cause an error
 
-const VALS_I16: (i16,) = //~ ERROR any use of this value will cause an error
+const VALS_I16: (i16,) =
     (
      i16::MAX + 1,
      );
+ //~^^ ERROR any use of this value will cause an error
 
-const VALS_I32: (i32,) = //~ ERROR any use of this value will cause an error
+const VALS_I32: (i32,) =
     (
      i32::MAX + 1,
      );
+ //~^^ ERROR any use of this value will cause an error
 
-const VALS_I64: (i64,) = //~ ERROR any use of this value will cause an error
+const VALS_I64: (i64,) =
     (
      i64::MAX + 1,
      );
+ //~^^ ERROR any use of this value will cause an error
 
-const VALS_U8: (u8,) = //~ ERROR any use of this value will cause an error
+const VALS_U8: (u8,) =
     (
      u8::MAX + 1,
      );
+ //~^^ ERROR any use of this value will cause an error
 
-const VALS_U16: (u16,) = ( //~ ERROR any use of this value will cause an error
+const VALS_U16: (u16,) = (
      u16::MAX + 1,
      );
+ //~^^ ERROR any use of this value will cause an error
 
-const VALS_U32: (u32,) = ( //~ ERROR any use of this value will cause an error
+const VALS_U32: (u32,) = (
      u32::MAX + 1,
      );
+ //~^^ ERROR any use of this value will cause an error
 
-const VALS_U64: (u64,) = //~ ERROR any use of this value will cause an error
+const VALS_U64: (u64,) =
     (
      u64::MAX + 1,
      );
+ //~^^ ERROR any use of this value will cause an error
 
 fn main() {
     foo(VALS_I8);
